/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen, Lenio
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson, Accenture
 *   Dennis Søgaard, Accenture
 *   Martin Bentzen, Accenture
 *   Jesper Jensen, Avanade
 *   Ramzi Fadel, Avanade
 *   Christian Pedersen, Accenture
 *   Morten Hougesen, Accenture
 *   Mikkel Hippe Brun, ITST
 *   Finn Hartmann Jordal, ITST
 *   Christian Lanng, ITST
 *
 */

package dk.gov.oiosi.common;

/**
 * A class modeling the .NET class TimeSpan.
 */
public class TimeSpan {

    /**
     * span in millis.
     */
    private long span;

    /**
     * Default Constructor with a time span of 0.
     */
    public TimeSpan() {
        span = 0;
    }

    /**
     * Constructs a TimeSpan of spanMillis milliseconds.
     * 
     * @param spanMillis The number of milliseconds in this time span
     */
    public TimeSpan(final long spanMillis) {
        span = spanMillis;
    }

    /**
     * Returns the timespan.
     * 
     * @return The time span in milliseconds.
     */
    public final long getSpanMillis() {
        return span;
    }

    /**
     * Set the timespan.
     * 
     * @param millis
     *            timespan.
     * @return timespan in millis.
     */
    public final TimeSpan setMillis(final long millis) {
        span = millis;
        return this;
    }

    /**
     * Set the timespan in seconds. eg. new TimeSpan().setSeconds( 42 );
     * 
     * @param seconds
     *            timespan.
     * @return the TimeSpan object.
     */
    public final TimeSpan setSeconds(final long seconds) {
        span = seconds * 1000;
        return this;
    }

    /**
     * Set the timespan in minuttes. eg. new TimeSpan().setMinuttes( 42 );
     * 
     * @param minuttes
     *            timespan.
     * @return the TimeSpan object.
     */
    public final TimeSpan setMinuttes(final long minuttes) {
        span = minuttes * 60 * 1000;
        return this;
    }

    /**
     * Set the timespan in hours. eg. new TimeSpan().setHours( 42 );
     * 
     * @param hours
     *            timespan.
     * @return the TimeSpan object.
     */
    public final TimeSpan setHours(final long hours) {
        span = hours * 60 * 60 * 1000;
        return this;
    }
}
