/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and 
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen, Lenio
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson, Accenture
 *   Dennis Søgaard, Accenture
 *   Martin Bentzen, Accenture
 *   Jesper Jensen, Avanade
 *   Ramzi Fadel, Avanade
 *   Christian Pedersen, Accenture
 *   Morten Hougesen, Accenture
 *   Mikkel Hippe Brun, ITST
 *   Finn Hartmann Jordal, ITST
 *   Christian Lanng, ITST
 *
 */


package dk.gov.oiosi.common.cache;

import dk.gov.oiosi.common.OutVariable;

/**
 * Interface that represents a simple cache 
 */
public interface ICache<TKey, TValue> {

	/**
     * Adds the key-value pair to the cache.
     * 
     * @param key
     *            key for cached item
     * @param value
     *            item to cache
     */
    void add(TKey key, TValue value);

    /**
     * Sets the value of the specified key. Overwrites value if already present.
     * 
     * @param key key for cached item
     * @param value item to cache
     */
    void set(TKey key, TValue value);
    
    /**
     * Lookup whether a given key is represented in the cache.
     * 
     * @param key
     *            key to search for
     * @return true if key exists
     */
    boolean containsKey(TKey key);

    /**
     * Removes a given key-value pair from the cache, based on the key.
     * 
     * @param key
     *            remove item for key
     */
    void remove(TKey key);

    /**
     * Try to get the value from a given key.
     * 
     * @param key
     *            search for item with key
     * @return value or null if unknown
     */
    boolean tryGetValue(TKey key, OutVariable<TValue> value);
}
