/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen, Lenio
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson, Accenture
 *   Dennis Søgaard, Accenture
 *   Martin Bentzen, Accenture
 *   Jesper Jensen, Avanade
 *   Ramzi Fadel, Avanade
 *   Christian Pedersen, Accenture
 *   Morten Hougesen, Accenture
 *   Mikkel Hippe Brun, ITST
 *   Finn Hartmann Jordal, ITST
 *   Christian Lanng, ITST
 *
 */

package dk.gov.oiosi.common.cache;

import dk.gov.oiosi.common.OutVariable;

/**
 * Dummy cache. Caches nothing.
 * 
 * Always returns false.
 */
public class TimedNullCache<TKey, TValue> implements ITimedCache<TKey, TValue> {

    /**
     * Do nothing.
     * @param key key for cached item
     * @param value item to cache
     */            
    public final void add(final TKey key, final TValue value) { }
    
    public final void set(final TKey key, final TValue value) { }

    /**
     * Always return false.
     * @param key key to search for
     * @return false always
     */
    public final boolean containsKey(final TKey key) {
        return false;
    }

    /**
     * Do nothing.
     * @param key remove item for key
     */
    public final void remove(final TKey key) { }

    /**
     * Always return null.
     * @param key
     *            search for item with key
     * @return always null
     */
    public final boolean tryGetValue(final TKey key, OutVariable<TValue> value) {
        return false;
    }
}
