/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and 
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen, Lenio
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson, Accenture
 *   Dennis Søgaard, Accenture
 *   Martin Bentzen, Accenture
 *   Jesper Jensen, Avanade
 *   Ramzi Fadel, Avanade
 *   Christian Pedersen, Accenture
 *   Morten Hougesen, Accenture
 *   Mikkel Hippe Brun, ITST
 *   Finn Hartmann Jordal, ITST
 *   Christian Lanng, ITST
 *
 */

package dk.gov.oiosi.communication.client;

import java.net.URL;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Vector;
import java.util.Map.Entry;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMNamespace;
import org.apache.axiom.om.impl.builder.StAXOMBuilder;
import org.apache.axiom.soap.SOAP12Constants;
import org.apache.axiom.soap.SOAPFactory;
import org.apache.axiom.soap.SOAPHeaderBlock;
import org.apache.axis2.AxisFault;
import org.apache.axis2.addressing.AddressingConstants;
import org.apache.axis2.addressing.EndpointReference;
import org.apache.axis2.client.Options;
import org.apache.axis2.client.ServiceClient;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.context.ConfigurationContextFactory;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.deployment.FileSystemConfigurator;
import org.apache.axis2.description.AxisService;
import org.apache.axis2.description.WSDL2Constants;
import org.apache.axis2.engine.AxisConfigurator;
import org.apache.axis2.transport.http.HTTPConstants;
import org.apache.axis2.transport.http.HttpTransportProperties.ProxyProperties;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.MultiThreadedHttpConnectionManager;
import org.apache.commons.httpclient.params.HttpConnectionManagerParams;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.neethi.Policy;
import org.apache.neethi.PolicyEngine;
import org.apache.ws.security.WSConstants;
import org.apache.ws.security.WSSecurityEngineResult;
import org.apache.ws.security.handler.WSHandlerConstants;
import org.apache.ws.security.handler.WSHandlerResult;
import org.wso2.mercury.callback.MercuryErrorCallback;
import org.wso2.mercury.client.MercuryClient;
import org.wso2.mercury.exception.RMException;
import org.wso2.mercury.util.MercuryClientConstants;
import org.wso2.mercury.util.RMUtil;

import dk.gov.oiosi.Constants;
import dk.gov.oiosi.addressing.Identifier;
import dk.gov.oiosi.communication.RMPolicy;
import dk.gov.oiosi.communication.configuration.mail.IMailServerConfiguration;
import dk.gov.oiosi.communication.configuration.mail.MailAuthenticationMode;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.security.SignatureValidationProof;

/**
 * Low level Dispatcher. Sets up Axis2 Stack.
 */
public class AxisMercuryDispatcher implements IAxisDispatcher {

	private static Log log = LogFactory.getLog(AxisMercuryDispatcher.class);
	
	private boolean useRM = true;
	private boolean useWSS = true;

	public void setUseRM(boolean v) {
		useRM = v;
	}

	public void setUseWSS(boolean v) {
		useWSS = v;
	}

	private RMPolicy httpRMPolicy = null;
	private RMPolicy mailRMPolicy = null;

	public void setHttpRMPolicy(RMPolicy policy) {
		httpRMPolicy = policy;
	}

	public void setMailRMPolicy(RMPolicy policy) {
		mailRMPolicy = policy;
	}

	private IMailServerConfiguration inboxMailServerConfiguration;
	
	@Override
	public void setInboxMailServerConfiguration(IMailServerConfiguration inboxMailServerConfiguration) {
		this.inboxMailServerConfiguration = inboxMailServerConfiguration;
	}
	
	/**
	 * Resolve resource file name.
	 */
	private String getFileName(String f) {
		try {
			URL url = AxisMercuryDispatcher.class.getClassLoader().getResource(
					f);
			return url.getFile();
		} catch (Exception e) {
			return f;
		}
	}

	private static String configContextKey = null;
	private static ConfigurationContext configContext = null;
	private static ConcurrentMap<String, Policy> wssPolicyMap = new ConcurrentHashMap<String, Policy>();
	private Policy wssPolicy = null;

	/*public static void registerConfigurationContext(String axisXml,
			String axisRepository, ConfigurationContext configurationContext) {
		configContextKey = "" + axisXml + "::" + axisRepository;
		configContext = configurationContext;
	}*/

	public AxisMercuryDispatcher(String axisXml, String axisRepository,
			String wssPolicyFile) throws Exception {
		String key = "" + axisXml + "::" + axisRepository;
		synchronized (AxisMercuryDispatcher.class) {
			// init Axis2 configuration context
			if (configContext == null) {
				configContextKey = "" + axisXml + "::" + axisRepository;

				log.info("Initialize Axis Configuration - axis2.xml : "
						+ axisXml + " - repository : " + axisRepository);
				AxisConfigurator axisConf = new FileSystemConfigurator(
						getFileName(axisRepository), getFileName(axisXml));
				configContext = ConfigurationContextFactory
						.createConfigurationContext(axisConf);
				/*
				 * Increase the number of default threads in Axis2 1.5.1
				 */
				MultiThreadedHttpConnectionManager multiThreadedHttpConnectionManager = new MultiThreadedHttpConnectionManager();

	            HttpConnectionManagerParams params = new HttpConnectionManagerParams();
	            params.setDefaultMaxConnectionsPerHost(200);
	            multiThreadedHttpConnectionManager.setParams(params);
	            HttpClient httpClient = new HttpClient(multiThreadedHttpConnectionManager);
	            configContext.setProperty(HTTPConstants.CACHED_HTTP_CLIENT, httpClient);
			} else {
				log.debug("Reuse Axis Configuration - axis2.xml : " + axisXml
						+ " - repository : " + axisRepository);
				if (!configContextKey.equals(key)) {
					log
							.warn("This Implementation only supports one configuration context");
				}
			}
		}

		// init wss policy
		wssPolicy = (Policy) wssPolicyMap.get(wssPolicyFile);
		if (wssPolicy == null) {
			log.info("Initialize WSS Policy from : " + wssPolicyFile);
			wssPolicy = loadPolicy(wssPolicyFile);
			wssPolicyMap.put(wssPolicyFile, wssPolicy);
		} else {
			log.debug("Reuse WSS Policy : " + wssPolicyFile);
		}

		httpRMPolicy = dk.gov.oiosi.configuration.ConfigurationHandler.getInstance().getSendPolicy().getHttpRMPolicy(); 
		mailRMPolicy = dk.gov.oiosi.configuration.ConfigurationHandler.getInstance().getSendPolicy().getMailRMPolicy(); 
	}

	private ProxyProperties proxy = null;

	/**
	 * Sets a proxy, through which the service call will be made. Useful for
	 * SOAP tracing.
	 */
	public void setProxy(String proxyHost, int proxyPort) {
		proxy = new ProxyProperties();
		proxy.setProxyName(proxyHost);
		proxy.setProxyPort(proxyPort);
	}

	/**
	 * @param doc
	 *            The XML document to be sent
	 * @param endpointAddress
	 *            The receiver's address
	 * @param action
	 *            The SOAP action (operation) to be used
	 * @param cert
	 *            The receivers public X509 certificate
	 * @param customHeaders
	 *            A list of custom SOAP headers to use
	 * @return The XML reply
	 * @throws Exception
	 */
	@Override
	public OMElement sendRaspRequest(OMElement doc, String endpointAddress,
			String action, X509Certificate cert,
			Map<String, Object> customHeaders, Map<String, Object> properties,
			SignatureValidationProof[] signatureValidationProof)
			throws Exception {

		String toEPR = endpointAddress;
		log.debug("Invoking against endpoint:" + toEPR);

		// Check whether this is a mail or http sending
		boolean mailtransport = toEPR.startsWith("mail");

		ServiceClient serviceClient = new ServiceClient(configContext, null);

		log.debug("Engaging modules..:");
		serviceClient.engageModule(org.apache.axis2.Constants.MODULE_ADDRESSING);
		serviceClient.engageModule("rasp");

		Options clientOptions = new Options();

		// Should we send through a proxy?
		if (proxy != null) {
			clientOptions.setProperty(HTTPConstants.PROXY, proxy);
		}

		// Do Basic Setup
		clientOptions
				.setSoapVersionURI(SOAP12Constants.SOAP_ENVELOPE_NAMESPACE_URI);
		clientOptions.setProperty(AddressingConstants.WS_ADDRESSING_VERSION,
				AddressingConstants.Final.WSA_NAMESPACE);

		// sync transfer?
		if (mailtransport) {
			//clientOptions.setUseSeparateListener(true);
			if (inboxMailServerConfiguration == null) {
				throw new MainException("Inbox configuration must be configured when using mail transport");
			}
			
			serviceClient.getAxisService().addParameter("transport.mail.Protocol","pop3");
			
			serviceClient.getAxisService().addParameter("transport.mail.Address", inboxMailServerConfiguration.getReplyAddress());
		    serviceClient.getAxisService().addParameter("transport.mail.replyToAddress", inboxMailServerConfiguration.getReplyAddress());
		    serviceClient.getAxisService().addParameter("transport.PollInterval", String.valueOf(inboxMailServerConfiguration.getConnectionPolicy().getPollingInterval().getSpanMillis() / 1000));
		    serviceClient.getAxisService().addParameter("mail.pop3.host", inboxMailServerConfiguration.getServerAddress());
		    serviceClient.getAxisService().addParameter("mail.pop3.port", String.valueOf(inboxMailServerConfiguration.getConnectionPolicy().getPort()));
		    serviceClient.getAxisService().addParameter("mail.pop3.user", inboxMailServerConfiguration.getUserName());
		    serviceClient.getAxisService().addParameter("mail.pop3.password",inboxMailServerConfiguration.getPassword());
		    
		    if (inboxMailServerConfiguration.getConnectionPolicy().getAuthenticationMode().equals(MailAuthenticationMode.SSL)) {
			    serviceClient.getAxisService().addParameter("mail.pop3.socketFactory.class", "javax.net.ssl.SSLSocketFactory");
			    serviceClient.getAxisService().addParameter("mail.pop3.socketFactory.fallback", "false");
			    serviceClient.getAxisService().addParameter("mail.pop3.socketFactory.port", String.valueOf(inboxMailServerConfiguration.getConnectionPolicy().getPort()));
		    }
		}
		else {
			clientOptions.setUseSeparateListener(false);
		}
		// anonymous sender
		String anonymous_reply_to_address = AddressingConstants.Final.WSA_ANONYMOUS_URL;
		clientOptions.setReplyTo(new EndpointReference(
				anonymous_reply_to_address));

		// set action
		clientOptions.setAction(action);

		// SET ENDPOINT
		clientOptions.setTo(new EndpointReference(toEPR));

		// SETUP RM
		String offeredSequenceId = "N/A";
		MercuryClient mc = null;

		if (useRM) {
			log.debug("Engaging Mercury .. : " + useRM);
			serviceClient.engageModule("Mercury");
			offeredSequenceId = RMUtil.getUUID();
			clientOptions.setProperty(MercuryClientConstants.INTERNAL_KEY,
					offeredSequenceId);

			RMPolicy rmPolicy;
			// transport specific setup..
			if (mailtransport) {
				rmPolicy = mailRMPolicy;
				clientOptions.setTransportInProtocol(org.apache.axis2.Constants.TRANSPORT_MAIL);
				
			} else {
				rmPolicy = httpRMPolicy;
				clientOptions.setTransportInProtocol(org.apache.axis2.Constants.TRANSPORT_HTTP);
			}
			
			long httpTimeout;
			if(rmPolicy.isExponentialBackoff())
				httpTimeout = getTotalTimeoutForRecursiveBackoff(rmPolicy.getMaximumRetransmissionCount(), rmPolicy.getRetransmissionInterval());
			else
				httpTimeout = rmPolicy.getRetransmissionInterval()*rmPolicy.getMaximumRetransmissionCount();

			clientOptions.setTimeOutInMilliSeconds(httpTimeout);
			
			mc = setupRM(serviceClient, rmPolicy);

		} else {
			serviceClient.disengageModule("Mercury");
		}

		// Setup Security
		if (useWSS) {
			log.debug("Engaging rampart module..:");
			serviceClient.engageModule("rampart");
			setupWSS(cert, serviceClient, clientOptions);
		} else {
			serviceClient.disengageModule("rampart");
		}

		setCustomHeaders(customHeaders, serviceClient, clientOptions);

		copyProperties(properties, clientOptions);

		OMElement reply = null;

		// On RM errors, we'd like to log them
		MercuryErrorCallback rmErrorCallback = new MercuryErrorCallback() {
			public void onError(RMException e) {
				log.info(e);
			}
		};
		clientOptions.setProperty(MercuryClientConstants.ERROR_CALLBACK,
				rmErrorCallback);

		// set options
		serviceClient.setOptions(clientOptions);

		reply = serviceClient.sendReceive(doc);

		// Extract the signature validation proof
		MessageContext msgCtxt = serviceClient.getLastOperationContext()
				.getMessageContext(WSDL2Constants.MESSAGE_LABEL_IN);
		signatureValidationProof[0] = (SignatureValidationProof) msgCtxt
				.getProperty(Constants.RASP_SIGNATURE_PROOF);

		// Clean up the transport
		serviceClient.cleanupTransport();
		
		// Shut down the communication
		mc.terminateSequence(offeredSequenceId);
		
		return reply;
	}

	public static void shutdownContext() throws Exception {
		synchronized (AxisMercuryDispatcher.class) {
			if (configContext == null) {
				// Do nothing, there is nothing to shutdown.
				log.warn("Shutdown method was called even though no configContext exists.");
			} else {
				configContext.terminate();
				configContext = null;
			}
		}
	}

	private void copyProperties(Map<String, Object> properties,
			Options clientOptions) {
		// Move Properties...
		if (properties != null && properties.size() > 0) {
			Set<String> enumeration = properties.keySet();
			for (String key : enumeration) {
				Object object = properties.get(key);
				clientOptions.setProperty(key, object);
			}
		}
	}

	private void setCustomHeaders(Map<String, Object> customHeaders,
			ServiceClient serviceClient, Options clientOptions) {
		// Custom headers
		serviceClient.removeHeaders();
		if (customHeaders != null) {
			log.debug("set Custom Headers");
			SOAPFactory fac = OMAbstractFactory.getSOAP12Factory();

			OMNamespace ns = fac.createOMNamespace(
					dk.gov.oiosi.Constants.RASP_CUSTOM_HEADER_NS,
					dk.gov.oiosi.Constants.RASP_CUSTOM_HEADER_PREFIX);

			List<SOAPHeaderBlock> headerList = new ArrayList<SOAPHeaderBlock>();
			for (Entry<String, Object> pair : customHeaders.entrySet()) {

				String key = pair.getKey();
				Object o = pair.getValue();

				SOAPHeaderBlock shb = null;
				if (o instanceof SOAPHeaderBlock) {
					shb = (SOAPHeaderBlock) o;
				} else {
					shb = fac.createSOAPHeaderBlock(key, ns);
					if (o instanceof Identifier) {
						Identifier identifier = (Identifier) o;
						shb.setText(identifier.getAsString());
						shb.addAttribute(
								dk.gov.oiosi.Constants.PARTY_IDENTIFIER_TYPE,
								identifier.getType().toLowerCase(), ns);
					} else {
						shb.setText(o.toString());
					}

				}
				if (dk.gov.oiosi.Constants.RASP_MESSAGE_IDENTIFIER.equals(key)) {
					clientOptions
							.setProperty(
									dk.gov.oiosi.Constants.RASP_CUSTOM_HEADER_MESSAGE_IDENTIFIER,
									shb);
				} else {
					if (dk.gov.oiosi.Constants.RASP_SENDER_PARTY_IDENTIFIER
							.equals(key)
							|| dk.gov.oiosi.Constants.RASP_RECEIVER_PARTY_IDENTIFIER
									.equals(key)) {
					}
					// For now 'RASP_CUSTOM_HEADER_MANDATORY' is only used
					// serverside - add to generel list...
					headerList.add(shb);
				}
			}
			// Add property with headers for RM Messages
			clientOptions.setProperty(
					dk.gov.oiosi.Constants.RASP_CUSTOM_HEADER_MANDATORY,
					headerList);

		} else {
			log.debug("No custom header");
		}
	}

	private void setupWSS(X509Certificate cert, ServiceClient serviceClient,
			Options clientOptions) {
		// set policies
		AxisService axisService = serviceClient.getServiceContext()
				.getAxisService();
		axisService.getPolicySubject().attachPolicy(wssPolicy);

		// set certificate to be used for encryption
		Vector<WSSecurityEngineResult> results = new Vector<WSSecurityEngineResult>();
		WSSecurityEngineResult wsser = new WSSecurityEngineResult(
				WSConstants.SIGN, null, cert, null, new byte[0]);
		results.add(wsser);
		WSHandlerResult wshr = new WSHandlerResult("STRING", results);
		Vector<WSHandlerResult> resultObj = new Vector<WSHandlerResult>();
		resultObj.add(wshr);
		clientOptions.setProperty(WSHandlerConstants.RECV_RESULTS, resultObj);
		log.debug("Receiver Certificate : " + cert.getSubjectDN() + " : "
				+ cert.getIssuerDN());
	}

	private MercuryClient setupRM(ServiceClient serviceClient, RMPolicy rmPolicy)
			throws AxisFault {
		MercuryClient mc;
		mc = new MercuryClient(serviceClient);
		mc.setRMSSequenceRetransmitTime(rmPolicy.getRetransmissionInterval());
		mc.setRMSMaximumRetransmitCount(rmPolicy
				.getMaximumRetransmissionCount());

		long timeoutValue = 0;
		if (rmPolicy.isExponentialBackoff()) {
			timeoutValue = getTotalTimeoutForRecursiveBackoff(rmPolicy
					.getMaximumRetransmissionCount(), rmPolicy.getRetransmissionInterval());

		} else {
			timeoutValue = rmPolicy.getRetransmissionInterval()
					* (rmPolicy.getMaximumRetransmissionCount() + 2);
		}

		// TODO: Find out about sleeper values
		mc.setRMSSequenceTimeout(timeoutValue);
		mc.setRMDSequenceTimeout(timeoutValue);
		mc.setInvokerTimeout(timeoutValue);

		log.debug("RM retransmission interval : "
				+ rmPolicy.getRetransmissionInterval() + " - count : "
				+ rmPolicy.getMaximumRetransmissionCount()
				+ " - exponentialBackup : " + rmPolicy.isExponentialBackoff()
				+ " - session timeout : " + timeoutValue);
		return mc;
	}

	private long getTotalTimeoutForRecursiveBackoff(int maxResends, long resendInterval){
		return resendInterval << maxResends + 1;
	}
	
	/**
	 * Helper for reading policy from file...
	 * 
	 * @param xmlPath
	 * @return
	 * @throws Exception
	 */
	private static Policy loadPolicy(String xmlPath) throws Exception {
		StAXOMBuilder builder = new StAXOMBuilder(xmlPath);
		Policy policy = PolicyEngine.getPolicy(builder.getDocumentElement());
		return policy;
	}
}
