/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen, Lenio
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson, Accenture
 *   Dennis Søgaard, Accenture
 *   Martin Bentzen, Accenture
 *   Jesper Jensen, Avanade
 *   Ramzi Fadel, Avanade
 *   Christian Pedersen, Accenture
 *   Morten Hougesen, Accenture
 *   Mikkel Hippe Brun, ITST
 *   Finn Hartmann Jordal, ITST
 *   Christian Lanng, ITST
 *
 */

package dk.gov.oiosi.exception;

import java.util.ArrayList;
import java.util.List;

import javax.xml.namespace.QName;

import org.apache.axiom.soap.SOAP11Constants;
import org.apache.axiom.soap.SOAP12Constants;
import org.apache.axis2.AxisFault;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.i18n.Messages;
import org.apache.commons.logging.LogFactory;

/**
 * Utility for creating and throwing Axis2 faults.
 */
public class AxisFaultUtil {

    /**
     * Build and throw Axis Fault.
     * 
     * @param msgContext
     *            message context
     * @param e
     *            exception
     * @throws AxisFault
     */
    public static void throwAxisFault(MessageContext msgContext, MainException e) throws AxisFault {
        AxisFault axisFault = buildAxisFault(msgContext, e);
        throw axisFault;
    }

    /**
     * Build and throw Axis Fault.
     * 
     * @param msgContext
     *            message context
     * @param soapFaultAction
     *            fault action
     * @param soapFaultCode
     *            fault code
     * @param e
     *            exception
     * @throws AxisFault
     */
    public static void throwAxisFault(MessageContext msgContext, String soapFaultAction, String soapFaultCode, Exception e)
            throws AxisFault {
        AxisFault axisFault = buildAxisFault(msgContext, soapFaultAction, soapFaultCode, e);
        throw axisFault;
    }

    /**
     * Build Axis Fault.
     * 
     * @param msgContext
     *            message context
     * @param e
     *            exception
     * @return AxisFault
     * @throws AxisFault
     */
    public static AxisFault buildAxisFault(MessageContext msgContext, MainException e) throws AxisFault {
        return buildAxisFault(msgContext, e.getSoapFaultAction(), e.getSoapFaultCode(), e);
    }

    /**
     * Build Axis Fault.
     * 
     * @param msgContext
     *            message context
     * @param soapFaultAction
     *            fault action
     * @param soapFaultCode
     *            fault code
     * @param e
     *            exception
     * @return AxisFault
     * @throws AxisFault
     */
    public static AxisFault buildAxisFault(MessageContext msgContext, String soapFaultAction, String soapFaultCode, Exception e) throws AxisFault {
        org.apache.commons.logging.Log log = LogFactory.getLog(AxisFaultUtil.class);
        log.debug("buildAxisFault : " + soapFaultAction + " : " + soapFaultCode );
        String faultActionUri = dk.gov.oiosi.Constants.RASP_NAME_SPACE + "/" + soapFaultAction;
        String nsURI = msgContext.getEnvelope().getNamespace().getNamespaceURI();
        QName soapFaultSubCode = new QName(nsURI, soapFaultAction);
        if (SOAP11Constants.SOAP_ENVELOPE_NAMESPACE_URI.equals(nsURI)) {
            //fac = OMAbstractFactory.getSOAP11Factory();

            AxisFault axisFault = new AxisFault( soapFaultSubCode, e.getMessage(), e );
            axisFault.setFaultAction( faultActionUri );
            throw axisFault;
        
        } else if (SOAP12Constants.SOAP_ENVELOPE_NAMESPACE_URI.equals(nsURI)) {
            //fac =OMAbstractFactory.getSOAP12Factory();
        } else {
            throw new AxisFault(Messages.getMessage("invalidSOAPversion"));
        }

        String message = e.getMessage();
        
        QName faultCode = new QName(SOAP12Constants.SOAP_ENVELOPE_NAMESPACE_URI, soapFaultCode, SOAP12Constants.SOAP_DEFAULT_NAMESPACE_PREFIX);

        List<QName> list = new ArrayList<QName>();
        list.add(new QName(dk.gov.oiosi.Constants.RASP_CUSTOM_HEADER_NS, soapFaultAction, dk.gov.oiosi.Constants.RASP_CUSTOM_HEADER_PREFIX));

        AxisFault tmp = new AxisFault(message, e);
        tmp.setStackTrace( e.getStackTrace() );
            
        tmp.setFaultAction(faultActionUri);
        tmp.setFaultCode( faultCode);
        tmp.setFaultSubCodes( list );

        // TODO : Get this fixed in Axis
        // THIS IS A TO AVOID A PROBLEM IN AXIS!!!
        // REUSE MERCURY AxisFaultUtil
        AxisFault axisFault = new AxisFault( message, org.wso2.mercury.util.AxisFaultUtil.generateAxisFaultMessageContext(msgContext, tmp) );
        axisFault.setStackTrace( e.getStackTrace() );

        log.debug("Set application fault type ..." );

        axisFault.setFaultType(org.apache.axis2.Constants.APPLICATION_FAULT);
        return axisFault;
    }


}
