/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen, Lenio
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson, Accenture
 *   Dennis Søgaard, Accenture
 *   Martin Bentzen, Accenture
 *   Jesper Jensen, Avanade
 *   Ramzi Fadel, Avanade
 *   Christian Pedersen, Accenture
 *   Morten Hougesen, Accenture
 *   Mikkel Hippe Brun, ITST
 *   Finn Hartmann Jordal, ITST
 *   Christian Lanng, ITST
 *
 */

package dk.gov.oiosi.security.revocation;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import dk.gov.oiosi.common.RaspLibraryException;
import dk.gov.oiosi.configuration.ConfigurationHandler;

/**
 * Creates an instance of a class with the IRevocationLookup interface.
 */
public class RevocationLookupFactory {

    private static Log log = LogFactory.getLog(RevocationLookupFactory.class);
    
    private static IRevocationLookup lookupClient;

    /**
     * Builds an instance of an IRevocationLookup client based on configuration.
     * 
     * @return revocation lookup
     */
    public static IRevocationLookup createRevocationLookupClient() throws Exception {

        synchronized (RevocationLookupFactory.class) {
            if (lookupClient == null) {
                RevocationLookupFactoryConfig config = ConfigurationHandler.getInstance().getRevocationLookupFactoryConfig();

                try {
                    log.debug("Init RevocationLookupFactory : " + config.getImplementationNamespaceClass());
                    lookupClient = (IRevocationLookup) Class.forName(config.getImplementationNamespaceClass()).newInstance();
                } catch (InstantiationException e) {
                    throw new RaspLibraryException(e);
                } catch (IllegalAccessException e) {
                    throw new RaspLibraryException(e);
                } catch (ClassNotFoundException e) {
                    throw new RaspLibraryException(e);
                }
            }
            return lookupClient;
        }

    }
}
