/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen, Lenio
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson, Accenture
 *   Dennis Søgaard, Accenture
 *   Martin Bentzen, Accenture
 *   Jesper Jensen, Avanade
 *   Ramzi Fadel, Avanade
 *   Christian Pedersen, Accenture
 *   Morten Hougesen, Accenture
 *   Mikkel Hippe Brun, ITST
 *   Finn Hartmann Jordal, ITST
 *   Christian Lanng, ITST
 *   Christian Uldall Pedersen (christian.u.pedersen@accenture.com)
 *
 */
package dk.gov.oiosi.security.revocation.crl;

import java.net.URL;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.util.Hashtable;

/**
 * Class used for storing CRLs retrieved from URL's in X509 certificates.
 */
class CrlCache
{
	private CertificateFactory cf;
	private Hashtable<URL, CrlInstance> table;
	
	public CrlCache() throws CertificateException
	{
		cf = CertificateFactory.getInstance("X.509");
		table = new Hashtable<URL, CrlInstance>();
	}
	
	/**
	 * Retrieves the CRL located at key.
	 * 
	 * @param key Get the CRL located at the URL "key"
	 * @return An instance of a CRL
	 */
	public synchronized CrlInstance getCRL(URL key)
	{
		if (table.containsKey(key))
		{
			return table.get(key);
		}
		else
		{
			CrlInstance instance = new CrlInstance(cf, key);
			table.put(key, instance);
			return instance;
		}
	}
}
