/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen, Lenio
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson, Accenture
 *   Dennis Søgaard, Accenture
 *   Martin Bentzen, Accenture
 *   Jesper Jensen, Avanade
 *   Ramzi Fadel, Avanade
 *   Christian Pedersen, Accenture
 *   Morten Hougesen, Accenture
 *   Mikkel Hippe Brun, ITST
 *   Finn Hartmann Jordal, ITST
 *   Christian Lanng, ITST
 *
 */

package dk.gov.oiosi.uddi;

import java.util.List;

import dk.gov.oiosi.exception.MainException;

/**
 * A Uddi lookup client is used for looking up endpoints in the UDDI.
 */
public interface IUddiLookupClient {

    /**
     * Looks up an endpoint as specified by the given parameters
     * 
     * @param parameters The parameters used for finding a specific endpoint
     * @return List of responses from the UDDI
     */
    List<UddiLookupResponse> lookup(LookupParameters parameters) throws MainException;


    /**
     * Gets the information about the process definitions from the UDDI.
     * 
     * @param processDefinitionIds The UDDI ids of the process definitions to get info about.
     * @return A list of process definitions.
     * @throws MainException 
     */
    List<ProcessDefinition> GetProcessDefinitions(List<UddiId> processDefinitionIds) throws MainException;
}
