/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen, Lenio
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson, Accenture
 *   Dennis Søgaard, Accenture
 *   Martin Bentzen, Accenture
 *   Jesper Jensen, Avanade
 *   Ramzi Fadel, Avanade
 *   Christian Pedersen, Accenture
 *   Morten Hougesen, Accenture
 *   Mikkel Hippe Brun, ITST
 *   Finn Hartmann Jordal, ITST
 *   Christian Lanng, ITST
 *
 */

package dk.gov.oiosi.xml.schematron;

import dk.gov.oiosi.xml.XmlDocument;

/**
 * Exception thrown when the schematron validation of an xml document fails.
 */
public class SchematronErrorException extends SchematronValidationException {

    /**
     * serialVersionUID.
     */
    private static final long serialVersionUID = 1L;

    /**
     * document holding the Schematron validation.
     */
    private XmlDocument schematronResult;

    /**
     * Gets the schematron result.
     * 
     * @return document with schematron error
     */
    public XmlDocument getSchematronResult() {
        return schematronResult;
    }

    /**
     * Constructor.
     * 
     * @param result
     *            The schematron result
     * @param firstErrorMessage
     *            The first error message
     */
    public SchematronErrorException(final XmlDocument result, final String firstErrorMessage) {
        super(firstErrorMessage);

        schematronResult = result;
    }

    /**
     * Constructor.
     * 
     * @param e
     *            message
     */
    public SchematronErrorException(final String e) {
        super(e);
    }
}
