package dk.gov.oiosi.test.common;

import junit.framework.Assert;
import junit.framework.TestCase;
import dk.gov.oiosi.addressing.Identifier;
import dk.gov.oiosi.addressing.IdentifierCvr;
import dk.gov.oiosi.addressing.IdentifierDuns;
import dk.gov.oiosi.addressing.IdentifierEan;
import dk.gov.oiosi.addressing.IdentifierIban;
import dk.gov.oiosi.addressing.IdentifierOvt;
import dk.gov.oiosi.addressing.IdentifierP;
import dk.gov.oiosi.addressing.IdentifierSe;
import dk.gov.oiosi.addressing.IdentifierVans;
import dk.gov.oiosi.common.Utilities;
import dk.gov.oiosi.communication.configuration.DocumentTypeConfig;
import dk.gov.oiosi.communication.configuration.ProfileMappingCollectionConfig;
import dk.gov.oiosi.configuration.ConfigurationException;
import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.test.TestConstants;
import dk.gov.oiosi.uddi.EndpointKeyTypeCode;
import dk.gov.oiosi.uddi.UddiId;
import dk.gov.oiosi.xml.XmlDocument;
import dk.gov.oiosi.xml.documenttype.DocumentTypeConfigSearcher;
import dk.gov.oiosi.xml.xpath.PrefixedNamespace;

public class UtilitiesTest extends TestCase {

	private DocumentTypeConfigSearcher _searcher;

	public UtilitiesTest() throws ConfigurationException {
		_searcher = new DocumentTypeConfigSearcher();
	}

	public void testProfileUddiIdCanBeResolvedFromValidOiosiMessage()
			throws Exception {
		String PATH_INVOICE_XML = "./test/xml/OIOUBL_Invoice_v2p1.xml";

		ConfigurationHandler
				.setConfigurationFile(TestConstants.PATH_TEST_RASPCONFIGURATION);
		ProfileMappingCollectionConfig profileMappingConfig = ConfigurationHandler
				.getInstance().getProfileMappingCollectionConfig();

		XmlDocument xmlDocument = new XmlDocument();
		xmlDocument.load(PATH_INVOICE_XML);

		DocumentTypeConfigSearcher documentTypeSearcher = new DocumentTypeConfigSearcher();
		DocumentTypeConfig documentTypeConfig = documentTypeSearcher
				.findUniqueDocumentType(xmlDocument);

		UddiId profileUddiId = Utilities.getProfileId(xmlDocument,
				documentTypeConfig, profileMappingConfig);
		String expectedProfileUddiId = "uddi:EBABEE8B-A5D3-4dc9-B976-3AAFF9A4E855";
		Assert.assertEquals(expectedProfileUddiId, profileUddiId.getID());
	}

	public void testIdentifierEAN() throws Exception {
		XmlDocument document = new XmlDocument();
		document.load(TestConstants.PATH_INVOICE_XML_IDENTIFIER_EAN);
		DocumentTypeConfig documentType = _searcher
				.findUniqueDocumentType(document);
		EndpointKeyTypeCode code = Utilities.getEndpointKeyTypeCode(
				document, documentType);
		Assert.assertEquals(EndpointKeyTypeCode.ean, code.getCode());
	
		String keyXpath = documentType.getEndpointType().getKey()
				.getXPath();
		PrefixedNamespace[] namespaces = documentType.getNamespaces();
	
		Identifier identifier = Utilities.getEndpointKeyByXpath(document,
				keyXpath, namespaces, code);
	
		Assert.assertEquals(identifier.getClass(), IdentifierEan.class);
	}

	
	public void testIdentifierOVT() throws Exception {
		XmlDocument document = new XmlDocument();
		document.load(TestConstants.PATH_INVOICE_XML_IDENTIFIER_OVT);
		DocumentTypeConfig documentType = _searcher
				.findUniqueDocumentType(document);
		EndpointKeyTypeCode code = Utilities.getEndpointKeyTypeCode(
				document, documentType);
		Assert.assertEquals(EndpointKeyTypeCode.ovt, code.getCode());

		String keyXpath = documentType.getEndpointType().getKey()
				.getXPath();
		PrefixedNamespace[] namespaces = documentType.getNamespaces();

		Identifier identifier = Utilities.getEndpointKeyByXpath(document,
				keyXpath, namespaces, code);

		Assert.assertEquals(identifier.getClass(), IdentifierOvt.class);
	}
	  
	  
	public void testIdentifierCVR() throws Exception {
		XmlDocument document = new XmlDocument();
		document.load(TestConstants.PATH_INVOICE_XML_IDENTIFIER_CVR);
		DocumentTypeConfig documentType = _searcher
				.findUniqueDocumentType(document);
		EndpointKeyTypeCode code = Utilities.getEndpointKeyTypeCode(
				document, documentType);
		Assert.assertEquals(EndpointKeyTypeCode.cvr, code.getCode());

		String keyXpath = documentType.getEndpointType().getKey()
				.getXPath();
		PrefixedNamespace[] namespaces = documentType.getNamespaces();

		Identifier identifier = Utilities.getEndpointKeyByXpath(document,
				keyXpath, namespaces, code);

		Assert.assertEquals(identifier.getClass(), IdentifierCvr.class);
	}
	  
	  
	public void testIdentifierP() throws Exception {
		XmlDocument document = new XmlDocument();
		document.load(TestConstants.PATH_INVOICE_XML_IDENTIFIER_P);
		DocumentTypeConfig documentType = _searcher
				.findUniqueDocumentType(document);
		EndpointKeyTypeCode code = Utilities.getEndpointKeyTypeCode(
				document, documentType);
		Assert.assertEquals(EndpointKeyTypeCode.p, code.getCode());

		String keyXpath = documentType.getEndpointType().getKey()
				.getXPath();
		PrefixedNamespace[] namespaces = documentType.getNamespaces();

		Identifier identifier = Utilities.getEndpointKeyByXpath(document,
				keyXpath, namespaces, code);

		Assert.assertEquals(identifier.getClass(), IdentifierP.class);
	}
	 
	public void testIdentifierSE() throws Exception {
		XmlDocument document = new XmlDocument();
		document.load(TestConstants.PATH_INVOICE_XML_IDENTIFIER_SE);
		DocumentTypeConfig documentType = _searcher
				.findUniqueDocumentType(document);
		EndpointKeyTypeCode code = Utilities.getEndpointKeyTypeCode(
				document, documentType);
		Assert.assertEquals(EndpointKeyTypeCode.se, code.getCode());

		String keyXpath = documentType.getEndpointType().getKey()
				.getXPath();
		PrefixedNamespace[] namespaces = documentType.getNamespaces();

		Identifier identifier = Utilities.getEndpointKeyByXpath(document,
				keyXpath, namespaces, code);

		Assert.assertEquals(identifier.getClass(), IdentifierSe.class);
	}
	  
	  
	public void testIdentifierVANS() throws Exception {
		XmlDocument document = new XmlDocument();
		document.load(TestConstants.PATH_INVOICE_XML_IDENTIFIER_VANS);
		DocumentTypeConfig documentType = _searcher
				.findUniqueDocumentType(document);
		EndpointKeyTypeCode code = Utilities.getEndpointKeyTypeCode(
				document, documentType);
		Assert.assertEquals(EndpointKeyTypeCode.vans, code.getCode());

		String keyXpath = documentType.getEndpointType().getKey()
				.getXPath();
		PrefixedNamespace[] namespaces = documentType.getNamespaces();

		Identifier identifier = Utilities.getEndpointKeyByXpath(document,
				keyXpath, namespaces, code);

		Assert.assertEquals(identifier.getClass(), IdentifierVans.class);
	}
	  
	  
	public void testIdentifierIBAN() throws Exception {
		XmlDocument document = new XmlDocument();
		document.load(TestConstants.PATH_INVOICE_XML_IDENTIFIER_IBAN);
		DocumentTypeConfig documentType = _searcher
				.findUniqueDocumentType(document);
		EndpointKeyTypeCode code = Utilities.getEndpointKeyTypeCode(
				document, documentType);
		Assert.assertEquals(EndpointKeyTypeCode.iban, code.getCode());

		String keyXpath = documentType.getEndpointType().getKey()
				.getXPath();
		PrefixedNamespace[] namespaces = documentType.getNamespaces();

		Identifier identifier = Utilities.getEndpointKeyByXpath(document,
				keyXpath, namespaces, code);

		Assert.assertEquals(identifier.getClass(), IdentifierIban.class);
	}
	 
	  
	public void testIdentifierDUNS() throws Exception {
		XmlDocument document = new XmlDocument();
		document.load(TestConstants.PATH_INVOICE_XML_IDENTIFIER_DUNS);
		DocumentTypeConfig documentType = _searcher
				.findUniqueDocumentType(document);
		EndpointKeyTypeCode code = Utilities.getEndpointKeyTypeCode(
				document, documentType);
		Assert.assertEquals(EndpointKeyTypeCode.duns, code.getCode());

		String keyXpath = documentType.getEndpointType().getKey()
				.getXPath();
		PrefixedNamespace[] namespaces = documentType.getNamespaces();

		Identifier identifier = Utilities.getEndpointKeyByXpath(document,
				keyXpath, namespaces, code);

		Assert.assertEquals(identifier.getClass(), IdentifierDuns.class);
	}
	 
}
