/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen, Lenio
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson, Accenture
 *   Dennis Soegaard, Accenture
 *   Martin Bentzen, Accenture
 *   Jesper Jensen, Avanade
 *   Ramzi Fadel, Avanade
 *   Christian Pedersen, Accenture
 *   Morten Hougesen, Accenture
 *   Mikkel Hippe Brun, ITST
 *   Finn Hartmann Jordal, ITST
 *   Christian Lanng, ITST
 *
 */

package dk.gov.oiosi.test.service;

import java.security.cert.X509Certificate;
import java.util.Map;

import org.apache.axiom.om.OMElement;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import dk.gov.oiosi.communication.OiosiMessage;
import dk.gov.oiosi.communication.configuration.DocumentTypeConfig;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.xml.XmlDocument;
import dk.gov.oiosi.xml.XmlSchema;
import dk.gov.oiosi.xml.documenttype.DocumentTypeConfigSearcher;
import dk.gov.oiosi.xml.schema.SchemaValidator;
import dk.gov.oiosi.xml.schematron.SchematronValidator;

/**
 * Service which receives real OIO/UBL documents.
 */
public class OiosiDocumentService extends OiosiService {

    /**
     * local log.
     */
    private static final Log log = LogFactory.getLog(OiosiService.class);

    /* (non-Javadoc)
     * @see dk.gov.oiosi.test.service.OiosiService#requestRespond(dk.gov.oiosi.communication.OiosiMessage)
     */
    public OiosiMessage requestRespond(OiosiMessage message) throws MainException {
        XmlDocument xmlDocument = message.getMessageXml();
        OMElement document = message.getMessageXml().getOMDocument();
        Map<String, Object> customHeaders = message.getMessageHeaders();
        String messageId = message.getMessageId();

        invokeCounter++;

        String msg = "Invokecount : total/confirm/timeout/cancel/abort = " + invokeCounter + "/" + confirmCounter + "/"
                + timeoutCounter + "/" + faultedCounter + " : " + messageId;
        String signingMessage = "Message was not signed ";
        X509Certificate cert = message.getSignerCertificate();
        if (cert != null) {
            signingMessage = "Message was signed with certificate : " + cert.getSubjectDN();
        }

        if (document == null) {
            log.info("Received empty document : " + msg);
        } else {
            log.info("Received document - try to detect : " + msg);
        }
        log.info(" - " + signingMessage);

        if (true) {
            log.info("ALWAYS - RETURN NULL AS REPLY");

            return null;
        }

        if (customHeaders != null) {
            log.info(" - service received custom headers ");
            for (String key : customHeaders.keySet()) {
                log.info(" - CustomKey : " + key + " : " + customHeaders.get(key));
            }
        } else {
            log.info(" - service received NO custom headers ");
        }

        if (document == null) {
            log.info(" - RETURN NULL AS REPLY");

            return null;
        }

        // Find Document
        DocumentTypeConfig documentTypeConfig = null;
        try {
            log.info("Find DocumentTypeConfig " + document.getLocalName());
            DocumentTypeConfigSearcher documentSearcher = new DocumentTypeConfigSearcher();
            documentTypeConfig = documentSearcher.findUniqueDocumentType(xmlDocument);
        } catch (MainException e) {
            throw e;
        } catch (Exception e) {
            log.error("Failed to find DocumentTypeConfig : " + e, e);
            throw new MainException("Failed to find DocumentTypeConfig ", e);
        }

        // Schema Check document
        try {
            log.info("SchemaValidate Document : " + documentTypeConfig.getSchemaPath());
            SchemaValidator validator = new SchemaValidator();

            // FileStream stream = File.OpenRead(xmlSchemaPath);
            XmlSchema schema = new XmlSchema(documentTypeConfig.getSchemaPath());

            validator.schemaValidateXmlDocument(xmlDocument, schema);
            log.info("- Schema validate OK ");
        } catch (MainException e) {
            log.error("Failed To Validate Schema : " + e);
            throw e;
        } catch (Exception e) {
            log.error("Failed To Validate Schema : " + e);
            throw new MainException("Failed To Validate Schema ", e);
        }

        // Schematron Check document
        try {
            log.info("Schematron Validate Document : " + documentTypeConfig.getSchemaPath());
            if (documentTypeConfig.getSchematronValidationConfig() == null) {
                // System.out.println(" - skip " );
            } else {
                log.info(" - doit ");
                SchematronValidator validator = new SchematronValidator(documentTypeConfig.getSchematronValidationConfig());

                validator.schematronValidateXmlDocument(xmlDocument);
                log.info("- Schematron validate OK ");
            }
        } catch (MainException e) {
            log.error("Failed To Schematron Validate document : " + e);
            throw e;
        } catch (Exception e) {
            log.error("Failed To Schematron Validate document : " + e);
            throw new MainException("Failed To Schematron Validate document", e);
        }

        log.info("Service handled message OK ");
        return null;
    }

}
