package dk.gov.oiosi.test.stubs;

import java.io.FileInputStream;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;

import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.exception.ArgumentNullException;
import dk.gov.oiosi.security.CertificateSubject;
import dk.gov.oiosi.security.lookup.CertificateLookupException;
import dk.gov.oiosi.security.lookup.CertificateNotFoundException;
import dk.gov.oiosi.security.lookup.ICertificateLookup;
import dk.gov.oiosi.security.lookup.MultipleCertificatesFoundException;
import dk.gov.oiosi.security.lookup.SearchFailedException;

public class LdapCertificateLookupStub implements ICertificateLookup {

	public X509Certificate getCertificate(CertificateSubject subjectSerialNumber)
			throws CertificateNotFoundException,
			MultipleCertificatesFoundException, SearchFailedException,
			CertificateLookupException {

		if(subjectSerialNumber == null) throw new ArgumentNullException("subjectSerialNumber");
		String fileName = ConfigurationHandler.getInstance().getLdapTestConfig();
		if(fileName == null) throw new SearchFailedException("No configuration found for the LDAP stub ");
		
		X509Certificate result;
		CertificateFactory factory;
		try {
			factory = CertificateFactory.getInstance("X509");
			result = (X509Certificate) factory.generateCertificate(new FileInputStream(fileName));
		} catch (Exception e) {
			throw new SearchFailedException(e.toString());
		}
		
        return result;

	}

}
