package dk.gov.oiosi.test.unit.common.cache;

import junit.framework.Assert;
import junit.framework.TestCase;
import dk.gov.oiosi.common.OutVariable;
import dk.gov.oiosi.common.cache.QuantityCache;

public class QuantityCacheTest extends TestCase {
    private QuantityCache<String, String> _cache;

    public void testSingleAddRemove() {
    	System.out.println("Single Add Remove Test Started");

        _cache = new QuantityCache<String, String>(4);
        _cache.add("test", "test");
        testExists("test");
        _cache.remove("test");
        testDoNotExists("test");

        System.out.println("Single Add Remove Test Completed");
    }

    public void testMultipleAddRemove() {
    	System.out.println("Multiple Add Remove Test Stated");

        _cache = new QuantityCache<String, String>(6);
        String[] a = new String[] { "a1", "a2", "a3" };
        String[] b = new String[] { "b1", "b2" };
        String[] c = new String[] { "c1", "c2", "c3" };

        addStrings(a);
        addStrings(b);
        checkStringsExists(a);
        checkStringsExists(b);

        removeStrings(b);
        checkStringsExists(a);
        checkStringsNotExists(b);

        addStrings(c);
        checkStringsExists(a);
        checkStringsExists(c);

        System.out.println("Multiple Add Remove Test Completed");
    }

    public void testMultipleAddAboveLimit() {
    	System.out.println("Multiple Add Above Limit Test Stated");

        _cache = new QuantityCache<String, String>(5);
        String[] a = new String[] { "a1", "a2", "a3" };
        String[] b = new String[] { "b1", "b2" };
        String[] c = new String[] { "c1", "c2", "c3" };

        addStrings(a);
        addStrings(b);
        checkStringsExists(a);
        checkStringsExists(b);

        addStrings(c);
        checkStringsExists(b);
        checkStringsExists(c);
        checkStringsNotExists(a);

        System.out.println("Multiple Add Above Limit Test Completed");
    }

    public void testMultipleAddAboveLimitLastViewed() {
    	System.out.println("Multiple Add Above Limit Last Viewed Test Stated");

        _cache = new QuantityCache<String, String>(4);
        String[] a = new String[] { "a1", "a2" };
        String[] b = new String[] { "b1", "b2" };
        String[] c = new String[] { "c1", "c2" };

        addStrings(a);
        addStrings(b);
        checkStringsExists(b);
        checkStringsExists(a);

        addStrings(c);
        checkStringsExists(a);
        checkStringsExists(c);
        checkStringsNotExists(b);

        System.out.println("Multiple Add Above Limit Last Viewed Test Completed");
    }
    
    /**
     * Test the set method
     */
    public void SetTest() {
        String a = "a";
        String b = "b";
        String c = "c";
        testSet(a, b);
        testSet(a, c);
        testElement(a, c);
        testRemove(a);
        testDoNotExists(a);
    }

    private void addStrings(String[] strings) {
    	for (String s : strings) {
    		testAdd(s, s);
    	}
    }

    private void removeStrings(String[] strings) {
    	for (String s : strings) {
    		testRemove(s);
    	}
    }

    private void checkStringsExists(String[] strings) {
    	for (String s : strings) {
    		testExists(s);
    	}
    }

    private void checkStringsNotExists(String[] strings) {
    	for (String s : strings) {
    		testDoNotExists(s);
    	}
    }

    private void testAdd(String key, String value) {
        _cache.add(key, value);
        testExists(key);
        testElement(key, value);
    }
    
    private void testSet(final String key, final String value) {
        _cache.set(key, value);
        testExists(key);
        testElement(key, value);
    }

    private void testRemove(String key) {
        _cache.remove(key);
        testDoNotExists(key);
    }

    private void testExists(String key) {
        Assert.assertTrue(_cache.containsKey(key));
        OutVariable<String> current = new OutVariable<String>();        
        Assert.assertTrue(_cache.tryGetValue(key, current));
    }

    private void testElement(String key, String expected) {
        OutVariable<String> current = new OutVariable<String>(); 
        Assert.assertTrue(_cache.tryGetValue(key, current));
        Assert.assertEquals(expected, current.getVariable());
    }

    private void testDoNotExists(String key) {
        Assert.assertFalse(_cache.containsKey(key));
        OutVariable<String> current = new OutVariable<String>(); 
        Assert.assertFalse(_cache.tryGetValue(key, current));
    }
}
