/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen, Lenio
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson, Accenture
 *   Dennis Soegaard, Accenture
 *   Martin Bentzen, Accenture
 *   Jesper Jensen, Avanade
 *   Ramzi Fadel, Avanade
 *   Christian Pedersen, Accenture
 *   Morten Hougesen, Accenture
 *   Mikkel Hippe Brun, ITST
 *   Finn Hartmann Jordal, ITST
 *   Christian Lanng, ITST
 *
 */

package dk.gov.oiosi.test.unit.security.oces;

import java.io.FileInputStream;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import junit.framework.TestCase;

import dk.gov.oiosi.common.OutVariable;
import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.security.CertificateSubject;
import dk.gov.oiosi.security.oces.OcesCertificateType;
import dk.gov.oiosi.security.oces.OcesX509Certificate;
import dk.gov.oiosi.test.TestConstants;

/**
 * test oces certificate types. TODO: find out what to test.
 */
public class OcesX509CertificateTest extends TestCase {

    private static Log log = LogFactory.getLog(OcesX509CertificateTest.class);


    static {
        ConfigurationHandler.setConfigurationFile(TestConstants.PATH_TEST_RASPCONFIGURATION);
    }

    /**
     * test oces certificate.
     */
    public void testEmployeeType() {
        log.info("testEmployeeType : " + TestConstants.PATH_CERTIFICATE_EMPLOYEE);
        try {
            String employeeCertificatePath = TestConstants.PATH_CERTIFICATE_EMPLOYEE;
            X509Certificate certificate = loadCertificate(employeeCertificatePath);
            OcesX509Certificate ocesCertificate = new OcesX509Certificate(certificate);
            assertTrue(OcesCertificateType.OcesEmployee == ocesCertificate.getOcesCertificateType().getType());
            log.info("Subject : " + ocesCertificate.getSubject().getSubjectString());
        } catch (Exception e) {
            log.error("Failed Employee Certificate : " + e, e);
            fail(e.getMessage());
        }
    }

    /**
     * test oces certificate.
     */
    public void testOrganisationType() {
        log.info("testOrganisationType : " + TestConstants.PATH_CERTIFICATE_ORGANISATION);
        try {
            String organisationCertificatePath = TestConstants.PATH_CERTIFICATE_ORGANISATION;
            X509Certificate certificate = loadCertificate(organisationCertificatePath);
            OcesX509Certificate ocesCertificate = new OcesX509Certificate(certificate);
            assertTrue(OcesCertificateType.OcesOrganisation == ocesCertificate.getOcesCertificateType().getType());
            log.info("Subject : " + ocesCertificate.getSubject().getSubjectString());
        } catch (Exception e) {
            log.error("Failed Organisation Certificate : " + e, e);
            fail(e.getMessage());
        }
    }

    /**
     * test oces certificate.
     */
    public void testFunctionType() {
        log.info("testDeviceType : " + TestConstants.PATH_CERTIFICATE_DEVICE);
        try {
            String deviceCertificatePath = TestConstants.PATH_CERTIFICATE_DEVICE;
            X509Certificate certificate = loadCertificate(deviceCertificatePath);
            OcesX509Certificate ocesCertificate = new OcesX509Certificate(certificate);
            assertTrue(OcesCertificateType.OcesDevice == ocesCertificate.getOcesCertificateType().getType());
            log.info("Subject : " + ocesCertificate.getSubject().getSubjectString());
        } catch (Exception e) {
            log.error("Failed Device Certificate : " + e, e);
            fail(e.getMessage());
        }
    }

    /**
     * test oces certificate.
     */
    public void testFunctionTestType() {
        log.info("testDeviceTestType : " + TestConstants.PATH_CERTIFICATE_DEVICE_TEST);
        try {
            String deviceCertificatePath = TestConstants.PATH_CERTIFICATE_DEVICE_TEST;
            X509Certificate certificate = loadCertificate(deviceCertificatePath);
            OcesX509Certificate ocesCertificate = new OcesX509Certificate(certificate);
            assertTrue(OcesCertificateType.OcesDevice == ocesCertificate.getOcesCertificateType().getType());
            log.info("Subject : " + ocesCertificate.getSubject().getSubjectString());
        } catch (Exception e) {
            log.error("Failed Device Certificate : " + e, e);
            fail(e.getMessage());
        }
    }

    /**
     * test oces certificate.
     */
    public void testPersonType() {
        log.info("testPersonType : " + TestConstants.PATH_CERTIFICATE_PERSON);
        try {
            String deviceCertificatePath = TestConstants.PATH_CERTIFICATE_PERSON;
            X509Certificate certificate = loadCertificate(deviceCertificatePath);
            OcesX509Certificate ocesCertificate = new OcesX509Certificate(certificate);
            assertTrue(OcesCertificateType.OcesPersonal == ocesCertificate.getOcesCertificateType().getType());
            log.info("Subject : " + ocesCertificate.getSubject().getSubjectString());
        } catch (Exception e) {
            log.error("Failed Person Certificate : " + e, e);
            fail(e.getMessage());
        }
    }

    /**
     * Illegal subject DN - contains spaces around '='. CertificateSubject should handle this.
     */
    public final static String SUBJECT_DN_ILLEGAL = "CN = service.ebx.dk (funktionscertifikat), SERIALNUMBER = CVR:25894375-FID:1200312847621, O = MYSUPPLY ApS // CVR:25894375, C = DK";

    /**
     * Legal subject DN.
     */
    public final static String SUBJECT_DN_OK = "CN=service.ebx.dk (funktionscertifikat), SERIALNUMBER=CVR:25894375-FID:1200312847621, O=MYSUPPLY ApS // CVR:25894375, C=DK";

    /**
     * Test illegal subject DN - contains spaces around '='. CertificateSubject should handle this.
     */
    public void testSubjectIllegal() {
        try {
            CertificateSubject subject = new CertificateSubject(SUBJECT_DN_ILLEGAL);
            log.info("SubjectSerialNumber = " + subject.getSerialNumberValue());
        } catch (Exception e) {
            log.error("Failed to parse subject : " + SUBJECT_DN_ILLEGAL + " : " + e, e);
            fail(e.getMessage());
        }
    }

    public void testSubjectOk() {
        try {
            CertificateSubject subject = new CertificateSubject(SUBJECT_DN_OK);
            log.info("SubjectSerialNumber = " + subject.getSerialNumberValue());
        } catch (Exception e) {
            log.error("Failed to parse subject : " + SUBJECT_DN_OK + " : " + e, e);
            fail(e.getMessage());
        }
    }

    public void testTryGetCvrOk() {
        try {
        	String deviceCertificatePath = TestConstants.PATH_CERTIFICATE_EMPLOYEE;
            X509Certificate certificate = loadCertificate(deviceCertificatePath);
            OcesX509Certificate ocesCertificate = new OcesX509Certificate(certificate);

            OutVariable<String> cvrNumberString = new OutVariable<String>();
			boolean success = ocesCertificate.TryGetCvrNumberString(cvrNumberString);
			assertTrue(success);
			assertEquals("25767535", cvrNumberString.getVariable());
        } catch (Exception e) {
            fail(e.getMessage());
        }
    }

    public void testTryGetCvrFail() {
        try {
        	String deviceCertificatePath = TestConstants.PATH_CERTIFICATE_PERSON;
            X509Certificate certificate = loadCertificate(deviceCertificatePath);
            OcesX509Certificate ocesCertificate = new OcesX509Certificate(certificate);

            OutVariable<String> cvrNumberString = new OutVariable<String>();
			boolean success = ocesCertificate.TryGetCvrNumberString(cvrNumberString);
			assertFalse(success);
        } catch (Exception e) {
            fail(e.getMessage());
        }
    }

    /**
     * util to load certificate.
     *
     * @param file
     *            filename
     * @return certificate
     * @throws Exception
     */
    private X509Certificate loadCertificate(String file) throws Exception {
        CertificateFactory factory = CertificateFactory.getInstance("X509");
        return (X509Certificate) factory.generateCertificate(new FileInputStream(file));
    }

}
