package dk.gov.oiosi.test.unit.uddi;

import junit.framework.Assert;

import org.junit.Test;

import dk.gov.oiosi.uddi.UddiGuidId;
import dk.gov.oiosi.uddi.UddiNonGuidId;


public class UddiGuidIDTest {
	
	@Test
	public void checkUddiGuidValidWithoutUddi() {
		Assert.assertFalse(UddiGuidId.isValidGuidId("uddi:4ba19475-2b75-452c-969c-fdeb5f07739e", false));
		Assert.assertTrue(UddiGuidId.isValidGuidId("4ba19475-2b75-452c-969c-fdeb5f07739e", false));
	}
	
	@Test
	public void checkUddiGuidValidWithUddi() {
		boolean notValid = UddiGuidId.isValidGuidId("4ba19475-2b75-452c-969c-fdeb5f07739e", true);
		boolean valid = UddiGuidId.isValidGuidId("uddi:4ba19475-2b75-452c-969c-fdeb5f07739e", true);
		Assert.assertFalse(notValid);
		Assert.assertTrue(valid);
	}
	
	@Test
	public void checkUddiGuidValidWithNemHandel() {
		boolean notValid = UddiGuidId.isValidGuidId("4ba19475-2b75-452c-969c-fdeb5f07739e", true);
		boolean valid = UddiGuidId.isValidGuidId("uddi:nemhandel.dk:4ba19475-2b75-452c-969c-fdeb5f07739e", true);
		Assert.assertFalse(notValid);
		Assert.assertTrue(valid);
	}
	
	@Test(expected = IllegalArgumentException.class) 
	public void checkNonUddiGuidWithoutUddi() {
		new UddiNonGuidId("4ba19475-2b75-452c-969c-fdeb5f07739e");
	}
	
	@Test(expected = IllegalArgumentException.class) 
	public void checkNonUddiGuidWithUddi() {
		new UddiNonGuidId("uddi:4ba19475-2b75-452c-969c-fdeb5f07739e");
	}
	
	@Test
	public void checkNonUddiGuid() {
		try {
			new UddiNonGuidId("uddi:4ba19475-2b75-452c-969c-fdeb5f07739e111111");
		} catch (IllegalArgumentException e) {
			Assert.fail();
		}
	}
	
}
