/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen, Lenio
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson, Accenture
 *   Dennis S�gaard, Accenture
 *   Martin Bentzen, Accenture
 *   Jesper Jensen, Avanade
 *   Ramzi Fadel, Avanade
 *   Christian Pedersen, Accenture
 *   Morten Hougesen, Accenture
 *   Mikkel Hippe Brun, ITST
 *   Finn Hartmann Jordal, ITST
 *   Christian Lanng, ITST
 *
 */

package dk.gov.oiosi.test.unit.xml.documenttype;

import java.io.FileNotFoundException;

import junit.framework.TestCase;
import dk.gov.oiosi.communication.configuration.DocumentTypeConfig;
import dk.gov.oiosi.configuration.ConfigurationException;
import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.test.TestConstants;
import dk.gov.oiosi.xml.XmlDocument;
import dk.gov.oiosi.xml.documenttype.NoDocumentTypeFoundFromXmlDocumentException;
import dk.gov.oiosi.xml.documenttype.DocumentTypeConfigSearcher;
import dk.gov.oiosi.xml.documenttype.SearchForDocumentTypeFromXmlDocumentFailedException;

/**
 * test document type config search.
 */
public class DocumentTypeConfigSearcherTest extends TestCase {
	private static final String friendlyName = "No Namespace";

	static {
		ConfigurationHandler
				.setConfigurationFile(TestConstants.PATH_TEST_RASPCONFIGURATION);
	}

	/**
	 * searcher.
	 */
	private static DocumentTypeConfigSearcher searcher;

	/**
	 * test init searcher.
	 * 
	 * @throws ConfigurationException
	 */
	public void testDocumentTypeConfigSearcher() throws ConfigurationException {
		searcher = new DocumentTypeConfigSearcher();
	}

	/**
	 * Searches for the application response 2.01 document type
	 * 
	 * @throws MainException
	 * @throws FileNotFoundException
	 */
	public void testSearchForApplicationResponse201()
			throws FileNotFoundException, MainException {
		assertFindDocument("Applikationsmeddelse",
				TestConstants.PATH_APPLICATIONRESPONSE201_XML);
	}

	public void testSearchForApplicationResponse202()
			throws FileNotFoundException, MainException {
		assertFindDocument("Applikationsmeddelse",
				TestConstants.PATH_APPLICATIONRESPONSE202_XML);
	}

	public void testSearchForCatalogue() throws FileNotFoundException,
			MainException {
		assertFindDocument("Katalog", TestConstants.PATH_CATALOGUE_XML);
	}

	public void testSearchForCatalogueDeletion() throws FileNotFoundException,
			MainException {
		assertFindDocument("Sletning af katalog",
				TestConstants.PATH_CATALOGUEDELETION_XML);
	}

	public void testSearchForCatalogueItemSpecificationUpdate()
			throws FileNotFoundException, MainException {
		assertFindDocument("Opdatering af katalogelement",
				TestConstants.PATH_CATALOGUEITEMSPECIFICATIONUPDATE_XML);
	}

	public void testSearchForCataloguePricingUpdate()
			throws FileNotFoundException, MainException {
		assertFindDocument("Opdatering af katalogpriser",
				TestConstants.PATH_CATALOGUEPRICINGUPDATE_XML);
	}

	public void testSearchForCatalogueRequest() throws FileNotFoundException,
			MainException {
		assertFindDocument("Katalogforespørgsel",
				TestConstants.PATH_CATALOGUEREQUEST_XML);
	}

	public void testSearchForCreditNote() throws FileNotFoundException,
			MainException {
		assertFindDocument("Kreditnota", TestConstants.PATH_CREDITNOTE_XML);
	}

	public void testSearchForInvoice() throws FileNotFoundException,
			MainException {
		assertFindDocument("Faktura", TestConstants.PATH_INVOICE_XML);
	}

	public void testSearchForOrder() throws FileNotFoundException,
			MainException {
		assertFindDocument("Ordre", TestConstants.PATH_ORDER_XML);
	}

	public void testSearchForOrderCancellation() throws FileNotFoundException,
			MainException {
		assertFindDocument("Ordreannulering",
				TestConstants.PATH_ORDERCANCELLATION_XML);
	}

	public void testSearchForOrderChange() throws FileNotFoundException,
			MainException {
		assertFindDocument("Ordreændring", TestConstants.PATH_ORDERCHANGE_XML);
	}

	public void testSearchForOrderResponse() throws FileNotFoundException,
			MainException {
		assertFindDocument("Ordrebekræftelse",
				TestConstants.PATH_ORDERRESPONSE_XML);
	}

	public void testSearchForOrderResponseSimple()
			throws FileNotFoundException, MainException {
		assertFindDocument("Simpel ordrebekræftelse",
				TestConstants.PATH_ORDERRESPONSESIMPLE_XML);
	}

	public void testSearchForReminder() throws FileNotFoundException,
			MainException {
		assertFindDocument("Rykker", TestConstants.PATH_REMINDER_XML);
	}

	public void testSearchForStatement() throws FileNotFoundException,
			MainException {
		assertFindDocument("KontoUdtog", TestConstants.PATH_STATEMENT_XML);
	}

	public void testSearchForInvoice07() throws FileNotFoundException,
			MainException {
		assertFindDocument("Faktura v0.7", TestConstants.PATH_INVOICE07_XML);
	}

	public void testSearchForNoNamespace() throws FileNotFoundException,
			MainException {
		assertFindDocument(friendlyName, TestConstants.PATH_NONAMESPACE_XML);
	}

	public void SearchForUnkownTypeTest() throws FileNotFoundException,
			MainException {
		assertFindDocument(null, TestConstants.PATH_UNKNOWNTYPE_XML);
	}

	public void NullArguementTest()
			throws NoDocumentTypeFoundFromXmlDocumentException,
			SearchForDocumentTypeFromXmlDocumentFailedException {
		searcher.findUniqueDocumentType(null);
	}

	private DocumentTypeConfig searchForDocument(String path)
			throws FileNotFoundException, MainException {
		XmlDocument document = new XmlDocument();
		document.load(path);
		DocumentTypeConfig documentType = searcher
				.findUniqueDocumentType(document);
		return documentType;
	}

	private void assertFindDocument(String friendlyName, String documentPath)
			throws FileNotFoundException, MainException {
		DocumentTypeConfig documentType = searchForDocument(documentPath);
		assertEquals(friendlyName, documentType.getFriendlyName());
	}
}
