/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen, Lenio
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson, Accenture
 *   Dennis S�gaard, Accenture
 *   Martin Bentzen, Accenture
 *   Jesper Jensen, Avanade
 *   Ramzi Fadel, Avanade
 *   Christian Pedersen, Accenture
 *   Morten Hougesen, Accenture
 *   Mikkel Hippe Brun, ITST
 *   Finn Hartmann Jordal, ITST
 *   Christian Lanng, ITST
 *
 */

package dk.gov.oiosi.test.unit.xml.schema;

import java.io.FileNotFoundException;

import junit.framework.TestCase;
import dk.gov.oiosi.communication.configuration.DocumentTypeConfig;
import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.test.TestConstants;
import dk.gov.oiosi.xml.XmlDocument;
import dk.gov.oiosi.xml.XmlSchema;
import dk.gov.oiosi.xml.documenttype.DocumentTypeConfigSearcher;
import dk.gov.oiosi.xml.schema.SchemaValidationFailedException;
import dk.gov.oiosi.xml.schema.SchemaValidator;
import dk.gov.oiosi.xml.schema.UnexpectedNamespaceException;

/**
 * perform schema validation tests on all oioubl documents
 */
public class SchemaValidationTest extends TestCase {

	static {
        ConfigurationHandler.setConfigurationFile(TestConstants.PATH_TEST_RASPCONFIGURATION);
    }
	
    private static DocumentTypeConfigSearcher docSearcher;

    public void testInitDocumentTypes() {
        try {
            docSearcher = new DocumentTypeConfigSearcher();
            assertTrue(true);
        }
        catch (MainException e) {
            fail(e.getMessage());
        }
    }

    public void testApplicationResponseValidation() throws Exception {
        schemaValidateDocument(TestConstants.PATH_APPLICATIONRESPONSE201_XML);
    }

    public void testCatalogueValidation() throws Exception {
        schemaValidateDocument(TestConstants.PATH_CATALOGUE_XML);
    }

    public void testCatalogueDeletionValidation() throws Exception {
        schemaValidateDocument(TestConstants.PATH_CATALOGUEDELETION_XML);
    }

    public void testCatalogueItemSpecificationUpdateValidation() throws Exception {
        schemaValidateDocument(TestConstants.PATH_CATALOGUEITEMSPECIFICATIONUPDATE_XML);
    }

    public void testCataloguePricingUpdateValidation() throws Exception {
        schemaValidateDocument(TestConstants.PATH_CATALOGUEPRICINGUPDATE_XML);
    }

    public void testCatalogueRequestValidation() throws Exception {
        schemaValidateDocument(TestConstants.PATH_CATALOGUEREQUEST_XML);
    }
    
    public void testCreditNoteValidation() throws Exception {
        schemaValidateDocument(TestConstants.PATH_CREDITNOTE_XML);
    }

    public void testInvoiceValidation() throws Exception {
        schemaValidateDocument(TestConstants.PATH_INVOICE_XML);
    }

    public void testOrderValidation() throws Exception {
        schemaValidateDocument(TestConstants.PATH_ORDER_XML);
    }

    public void testOrderCancellationValidation() throws Exception {
        schemaValidateDocument(TestConstants.PATH_ORDERCANCELLATION_XML);
    }

    public void testOrderChangeValidation() throws Exception {
        schemaValidateDocument(TestConstants.PATH_ORDERCHANGE_XML);
    }

    public void testOrderResponseValidation() throws Exception {
        schemaValidateDocument(TestConstants.PATH_ORDERRESPONSE_XML);
    }

    public void testOrderResponseSimpleValidation() throws Exception {
        schemaValidateDocument(TestConstants.PATH_ORDERRESPONSESIMPLE_XML);
    }

    public void testReminderValidation() throws Exception {
        schemaValidateDocument(TestConstants.PATH_REMINDER_XML);
    }

    public void testStatementValidation() throws Exception {
        schemaValidateDocument(TestConstants.PATH_STATEMENT_XML);
    }
    
    public void testUtilityStatementValidation() throws Exception {
        schemaValidateDocument(TestConstants.PATH_UTILITYSTATEMENT_XML);
    }

    public void testInvoice07Validation() throws Exception {
        schemaValidateDocument(TestConstants.PATH_INVOICE07_XML);
    }

    public void testCreditNote07Validation() throws Exception {
        schemaValidateDocument(TestConstants.PATH_CREDITNOTE07_XML);
    }

    public void testInvoiceWrongNamespaceValidation() throws Exception {
        try {
            String xmlPath = TestConstants.PATH_INVOICEWRONGNAMESPACE_XML;
            String schemaPath = TestConstants.PATH_INVOICE_XSD;
            schemaValidateDocument(xmlPath, schemaPath); // _validator201);
            fail("This should fail");
        }
        catch (SchemaValidationFailedException e) {
            assertTrue(true);
        }
        catch (UnexpectedNamespaceException e) {
            assertTrue(true);
        }
        catch (Exception e) {
            fail("This should fail" + e);
        }
    }

    public void testInvoiceWrongElementValidation() throws Exception {
        try {
            String xmlPath = TestConstants.PATH_INVOICEWRONGELEMENT_XML;
            String schemaPath = TestConstants.PATH_INVOICE_XSD;
            schemaValidateDocument(xmlPath, schemaPath);
            assertTrue(false);
        }
        catch (SchemaValidationFailedException e) {
            assertTrue(true);
        }
        catch (Exception e) {
            fail(e.getMessage());
        }
    }

    private void schemaValidateDocument(String xmlDocumentPath) throws SchemaValidationFailedException, MainException, FileNotFoundException {
        SchemaValidator validator = new SchemaValidator();

        XmlDocument document = new XmlDocument();
        document.load(xmlDocumentPath);

        DocumentTypeConfig documentType = docSearcher.findUniqueDocumentType(document);
        String xmlSchemaPath = documentType.getSchemaPath();
       
        XmlSchema schema = new XmlSchema(xmlSchemaPath);
        validator.schemaValidateXmlDocument(document, schema);
    }

    private void schemaValidateDocument(String xmlDocumentPath, String xmlSchemaPath) throws SchemaValidationFailedException, MainException, FileNotFoundException {
        SchemaValidator validator = new SchemaValidator();

        XmlDocument document = new XmlDocument();
        document.load(xmlDocumentPath);
       
        XmlSchema schema = new XmlSchema(xmlSchemaPath);
        validator.schemaValidateXmlDocument(document, schema);
    }

}
