/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen, Lenio
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson, Accenture
 *   Dennis S�gaard, Accenture
 *   Martin Bentzen, Accenture
 *   Jesper Jensen, Avanade
 *   Ramzi Fadel, Avanade
 *   Christian Pedersen, Accenture
 *   Morten Hougesen, Accenture
 *   Mikkel Hippe Brun, ITST
 *   Finn Hartmann Jordal, ITST
 *   Christian Lanng, ITST
 *
 */

package dk.gov.oiosi.test.unit.xml.schematron;

import java.io.FileNotFoundException;

import junit.framework.TestCase;

import dk.gov.oiosi.communication.configuration.DocumentTypeConfig;
import dk.gov.oiosi.configuration.ConfigurationHandler;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.test.TestConstants;
import dk.gov.oiosi.xml.XmlDocument;
import dk.gov.oiosi.xml.documenttype.DocumentTypeConfigSearcher;
import dk.gov.oiosi.xml.schematron.SchematronErrorException;
import dk.gov.oiosi.xml.schematron.SchematronValidationConfig;
import dk.gov.oiosi.xml.schematron.SchematronValidator;


/**
 * Runs a schematron validation on all document types
 */
public class SchematronValidationTest extends TestCase {

	static {
		 ConfigurationHandler.setConfigurationFile(TestConstants.PATH_TEST_RASPCONFIGURATION);
	}
	 
    public void testInitDocumentTypes() throws Exception {
    	new DocumentTypeConfigSearcher();
    }

    public void testOioublApplicationResponseValidation() throws Exception {
        validate(TestConstants.PATH_APPLICATIONRESPONSE201_XML);
    }

    public void testOioublCatalogueValidation() throws Exception {
        validate(TestConstants.PATH_CATALOGUE_XML);
    }

    public void testOioublCatalogueDeletionValidation() throws Exception {
        validate(TestConstants.PATH_CATALOGUEDELETION_XML);
    }

    public void testOioublCatalogueItemSpecificationUpdateValidation() throws Exception {
        validate(TestConstants.PATH_CATALOGUEITEMSPECIFICATIONUPDATE_XML);
    }

    public void testOioublCataloguePricingUpdateValidation() throws Exception {
        validate(TestConstants.PATH_CATALOGUEPRICINGUPDATE_XML);
    }

    public void testOioublCatalogueRequestValidation() throws Exception {
        validate(TestConstants.PATH_CATALOGUEREQUEST_XML);
    }

    public void testOioublCreditNoteValidation() throws Exception {
        validate(TestConstants.PATH_CREDITNOTE_XML);
    }

    public void testOioublInvoiceValidation() throws Exception {
        validate(TestConstants.PATH_INVOICE_XML);
    }

    public void testOioublOrderValidation() throws Exception {
        validate(TestConstants.PATH_ORDER_XML);
    }

    public void testOioublOrderCancellationValidation() throws Exception {
        validate(TestConstants.PATH_ORDERCANCELLATION_XML);
    }
    
    public void testOioublOrderChangeValidation() throws Exception {
        validate(TestConstants.PATH_ORDERCHANGE_XML);
    }

    public void testOioublOrderResponseValidation() throws Exception {
        validate(TestConstants.PATH_ORDERRESPONSE_XML);
    }

    public void testOioublOrderResponseSimpleValidation() throws Exception {
        validate(TestConstants.PATH_ORDERRESPONSESIMPLE_XML);
    }

    public void testOioublReminderValidation() throws Exception {
        validate(TestConstants.PATH_REMINDER_XML);
    }

    public void testOioublStatementValidation() throws Exception {
        validate(TestConstants.PATH_STATEMENT_XML);
    }
    
    public void testOioublUtilityStatementValidation() throws Exception {
        validate(TestConstants.PATH_UTILITYSTATEMENT_XML);
    }

    public void testOioxmlInvoiceValidation07() throws Exception {
        validate(TestConstants.PATH_INVOICE07_XML);
    }

    public void testOioxmlCreditNoteValidation07() throws Exception {
        validate(TestConstants.PATH_CREDITNOTE07_XML);
    }

    public void testOioublNoIdInvoiceValidation() throws Exception {
        try {
            validate(TestConstants.PATH_INVOICENOID_XML);
            fail("testOioublNoIdInvoiceValidation should fail");
        }
        catch (SchematronErrorException e) {
            assertTrue(true);
        }
    }

    public void testOioxmlInvalidEanNumberInvoiceValidation() throws Exception {
        try {
            validate(TestConstants.PATH_INVOICEN7INVALIDEANNUMBER_XML);
            fail("testOioxmlInvalidEanNumberInvoiceValidation should give error");
        }
        catch (SchematronErrorException e) {
            assertTrue(true);
        }
    }

    /**
     * perform validation.
     * 
     * @param xmlDocumentPath
     *            path to document
     * @throws FileNotFoundException 
     * @throws MainException 
     */
    private void validate(String xmlDocumentPath) throws FileNotFoundException, MainException {
        XmlDocument document = new XmlDocument();
        document.load(xmlDocumentPath);
        DocumentTypeConfigSearcher docSearcher = new DocumentTypeConfigSearcher();
        DocumentTypeConfig documentType = docSearcher.findUniqueDocumentType(document);
        SchematronValidationConfig schematronValidationConfig = documentType.getSchematronValidationConfig();
        SchematronValidator validator = new SchematronValidator(schematronValidationConfig);
        validator.schematronValidateXmlDocument(document);
        schematronValidationConfig = null;
        validator = null;
        document = null;
    }
}
